# edl : common library for the energy-dashboard tool-chain
# Copyright (C) 2019  Todd Greenwood-Geer (Enviro Software Solutions, LLC)
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""
(edc-cli) toddg@beast:~/proj/energy-dashboard$ edc feed data-oasis-atl-ruc-zone-map proc insert
{"ts":"09/18/2019 04:12:54 PM", "msg":{"name": "edl.resources.db", "src": "data-oasis-atl-ruc-zone-map", "method": "insert", "sql_dir": "/home/toddg/proj/energy-dashboard/data/data-oasis-atl-ruc-zone-map/sql", "db_dir": "/home/toddg/proj/energy-dashboard/data/data-oasis-atl-ruc-zone-map/db", "new_files": 99}}
Traceback (most recent call last):
  File "src/40_inse.py", line 98, in <module>
    run(logger, m, config())
  File "src/40_inse.py", line 78, in run
    state_file)
  File "/home/toddg/bin/anaconda3/envs/edc-cli/lib/python3.7/site-packages/edl/resources/state.py", line 22, in update
    for item in generator:
  File "/home/toddg/bin/anaconda3/envs/edc-cli/lib/python3.7/site-packages/edl/resources/db.py", line 34, in insert
    yield insert_file(logger, resource_name, sql_dir, db_dir, sql_file_name, idx, depth=0, max_depth=5)
  File "/home/toddg/bin/anaconda3/envs/edc-cli/lib/python3.7/site-packages/edl/resources/db.py", line 56, in insert_file
    with sqlite3.connect(os.path.join(db_dir, db_name)) as cnx:
sqlite3.OperationalError: unable to open database file
"""


import os
import logging
import sqlite3
from edl.resources import log

def insert(logger, resource_name, sql_dir, db_dir, new_files):
    chlogger = logger.getChild(__name__)
    new_files_count = len(new_files)
    if not os.path.exists(db_dir):
        os.makedirs(db_dir)
    if not os.path.exists(db_dir):
        raise Exception("Failed to create db_dir: %s" % db_dir)
    log.info(chlogger, {
        "name"      : __name__,
        "src"       : resource_name, 
        "method"    : "insert",
        "sql_dir"   : sql_dir,
        "db_dir"    : db_dir,
        "new_files" : new_files_count,
        })
    for (idx, sql_file_name) in enumerate(new_files):
        yield insert_file(logger, resource_name, sql_dir, db_dir, sql_file_name, idx, depth=0, max_depth=5)

def insert_file(logger, resource_name, sql_dir, db_dir, sql_file_name, idx, depth, max_depth):
    chlogger = logger.getChild(__name__)
    db_name = gen_db_name(resource_name, depth)
    sql_file = os.path.join(sql_dir, sql_file_name)
    if depth > max_depth:
        log.error(chlogger, {
            "name"      : __name__,
            "src"       : resource_name, 
            "method"    : "insert",
            "sql_dir"   : sql_dir,
            "db_dir"    : db_dir,
            "db_name"   : db_name,
            "file_idx"  : idx,
            "sql_file"  : sql_file,
            "depth"     : depth,
            "max_depth" : max_depth,
            "ERROR"     :"insert sql_file failed, max_depth exceeded",
            })
        return
        
    with sqlite3.connect(os.path.join(db_dir, db_name)) as cnx:
        try:
            with open(sql_file, 'r') as sf:
                log.info(chlogger, {
                    "name"      : __name__,
                    "src"       : resource_name, 
                    "method"    : "insert",
                    "sql_dir"   : sql_dir,
                    "db_dir"    : db_dir,
                    "db_name"   : db_name,
                    "file_idx"  : idx,
                    "sql_file"  : sql_file,
                    "depth"     : depth,
                    "message"   : "started",
                    })
                cnx.executescript(sf.read())
                log.info(chlogger, {
                    "name"      : __name__,
                    "src"       : resource_name, 
                    "method"    : "insert",
                    "sql_dir"   : sql_dir,
                    "db_dir"    : db_dir,
                    "db_name"   : db_name,
                    "file_idx"  : idx,
                    "sql_file"  : sql_file,
                    "depth"     : depth,
                    "message"   : "completed",
                    })
            return sql_file
        except Exception as e:
            log.error(chlogger, {
                "name"      : __name__,
                "src"       : resource_name, 
                "method"    : "insert",
                "sql_dir"   : sql_dir,
                "db_dir"    : db_dir,
                "db_name"   : db_name,
                "file_idx"  : idx,
                "sql_file"  : sql_file,
                "depth"     : depth,
                "ERROR":"insert sql_file failed",
                "exception": str(e),
                })
            insert_file(logger, resource_name, sql_dir, db_dir, sql_file, idx, depth+1, max_depth)

def gen_db_name(resource_name, depth):
    return "%s_%02d.db" % (resource_name, depth)



# edl : common library for the energy-dashboard tool-chain
# Copyright (C) 2019  Todd Greenwood-Geer (Enviro Software Solutions, LLC)
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import logging
import zipfile as zf

def unzip(resource_name, zip_files, input_dir, output_dir):
    """
    Unzip the zip files in input dir to the output directory.

    Return a list of unzipped artifacts that will later be appended
    to the state_file.
    """
    for f in zip_files:
        yield unzip_file(f, resource_name, input_dir, output_dir)

def unzip_file(f, resource_name, input_dir, output_dir):
    try:
        with zf.ZipFile(os.path.join(input_dir, f), 'r') as t:
            for zip_item in t.namelist():
                target_artifact = os.path.join(output_dir, zip_item)
                if not os.path.exists(target_artifact):
                    t.extract(zip_item, output_dir)
                    logging.info({
                        "src":resource_name, 
                        "action":"unzip",
                        "zip_file":f,
                        "zip_item":zip_item,
                        "msg": "item extracted"})
                else:
                    logging.info({
                        "src":resource_name, 
                        "action":"unzip",
                        "zip_file":f,
                        "zip_item":zip_item,
                        "msg": "item skipped (exists already)"})
            return f
    except Exception as e:
        logging.error({
            "src":resource_name, 
            "action":"new_zip_files",
            "file":f,
            "error": e
            })
        return ""



# edl : common library for the energy-dashboard tool-chain
# Copyright (C) 2019  Todd Greenwood-Geer (Enviro Software Solutions, LLC)
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import logging
from edl.resources import filesystem

def update(generator, state_file):
    for item in generator:
        with open(state_file, "a") as f:
            f.write("%s\n" % item)

def new_files(resource_name, state_file, path, ending):
    """Return a list of files that are not present in the state file""" 
    processed_file_set = set()
    logging.info({
        "src":resource_name, 
        "action":"new_%s_files" % ending})
    if os.path.exists(state_file):
        with open(state_file, 'r') as m:
            processed_file_set = set([l.rstrip() for l in m])
    all_file_set = set(filesystem.glob_dir(path, ending))
    new_file_set = all_file_set - processed_file_set
    logging.info({
        "src":resource_name, 
        "action":"new_%s_files" % ending, 
        "new_file_set_count":len(new_file_set), 
        "all_file_set_count": len(all_file_set), 
        "processed_file_set_count": len(processed_file_set)})
    return list(new_file_set)



# edl : common library for the energy-dashboard tool-chain
# Copyright (C) 2019  Todd Greenwood-Geer (Enviro Software Solutions, LLC)
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import io
import subprocess
import time

def runyield(cmd, cwd):
    filename    = 'edc.log'
    with io.open(filename, 'wb') as writer, io.open(filename, 'rb', 1) as reader:
        process = subprocess.Popen(cmd, cwd=cwd, shell=True, stdout=writer)
        while process.poll() is None:
            data = reader.read()
            if len(data) > 0:
                yield reader.read()
            time.sleep(0.1)
        yield reader.read()




# edl : common library for the energy-dashboard tool-chain
# Copyright (C) 2019  Todd Greenwood-Geer (Enviro Software Solutions, LLC)
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""
time.py : compute things like date ranges
"""
import datetime

def day_range(start_date, end_date):
    return sorted([start_date + datetime.timedelta(n) 
        for n in range(int ((end_date - start_date).days))])

def day_range_to_today(start_date):
    return day_range(start_date, datetime.datetime.now().date())

def range_pairs(dates):
    start = dates[:-1]
    end = dates[1:]
    return zip(start,end)



# -*- coding: utf-8 -*-
"""Example setup.py file

This example setup file is adapted from https://github.com/pypa/sampleproject
and is not supposed to be and exahustive list of options accepted by the
setuptools' function `setup`. You should read the respective documentation,
which can be found at https://setuptools.readthedocs.io/en/latest/setuptools.html

The original License and Copyright for this setup file follows:

Copyright (c) 2016 The Python Packaging Authority (PyPA)

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the README file
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    # This is the name of your project. The first time you publish this
    # package, this name will be registered for you. It will determine how
    # users can install this project, e.g.:
    #
    # $ pip install sampleproject
    #
    # And where it will live on PyPI: https://pypi.org/project/sampleproject/
    #
    # There are some restrictions on what makes a valid project name
    # specification here:
    # https://packaging.python.org/specifications/core-metadata/#name
    name='energy-dashboard-library',  # Required

    # Versions should comply with PEP 440:
    # https://www.python.org/dev/peps/pep-0440/
    #
    # For a discussion on single-sourcing the version across setup.py and the
    # project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.11',  # Required

    # This is a one-line description or tagline of what your project does. This
    # corresponds to the "Summary" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#summary
    description='''Energy Dashboard Library''',  # Required

    # This is an optional longer description of your project that represents
    # the body of text which users will see when they visit PyPI.
    #
    # Often, this is the same as your README, so you can just read it in from
    # that file directly (as we have already done above)
    #
    # This field corresponds to the "Description" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-optional
    long_description=long_description,  # Optional

    # Denotes that our long_description is in Markdown; valid values are
    # text/plain, text/x-rst, and text/markdown
    #
    # Optional if long_description is written in reStructuredText (rst) but
    # required for plain-text or Markdown; if unspecified, "applications should
    # attempt to render [the long_description] as text/x-rst; charset=UTF-8 and
    # fall back to text/plain if it is not valid rst" (see link below)
    #
    # This field corresponds to the "Description-Content-Type" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-content-type-optional
    long_description_content_type='text/markdown',  # Optional (see note above)

    # This should be a valid link to your project's main homepage.
    #
    # This field corresponds to the "Home-Page" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#home-page-optional
    url='http://github.com/energy-analytics-project/energy-dashboard-lib',  # Optional

    # This should be your name or the name of the organization which owns the
    # project.
    author='Todd Greenwood-Geer (Enviro Software Solutions, LLC)',  # Optional

    # This should be a valid email address corresponding to the author listed
    # above.
    author_email='pub+github@zwrob.com',  # Optional

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        #'Programming Language :: Python :: 2',
        #'Programming Language :: Python :: 2.7',
        #'Programming Language :: Python :: 3',
        #'Programming Language :: Python :: 3.4',
        #'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(),  # Required

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html

    # NOTE: not using xmltodict until https://github.com/martinblech/xmltodict/pull/224 get's accepted.
    # 'xmltodict==0.12.0',
    install_requires=[
            'idna==2.8',
            'requests==2.22.0',
            'urllib3==1.25.3',
            'requests',
            "Jinja2",
    ],  
    # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `edl`
    # which executes the function `main` from this package when invoked:
    entry_points={  # Optional
        'console_scripts': [
        ],
    }
)
